// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Tools/Logging.hh"
#include "Rivet/Projections/ChargedFinalState.hh"
#include "Rivet/Math/Units.hh"
#include <vector>

using namespace std;

namespace Rivet {


  class  LHCB_2015_I1333223 : public Analysis {
  public:

    /// @name Constructors etc.
    //@{

    /// Constructor
     LHCB_2015_I1333223()
      : Analysis("LHCB_2015_I1333223"),
        _etaMin(2.0),
        _etaMax(4.5),
        _ptMin(0.2)
    {    }

    //@}


  public:

    /// @name Analysis methods
    //@{

    /// Book histograms and initialise projections before the run
    void init() {
      // Initialise and register projections here
      addProjection(ChargedFinalState(_etaMin, _etaMax, _ptMin*GeV), "CFS");

      // Book histograms here, e.g.:
      // Reproducing only measurement for prompt charged particles
      _hInelasticXs = bookHisto1D(1, 1, 1);

    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {
      const double             weight  = event.weight();
      const ChargedFinalState   &cfs    = applyProjection<ChargedFinalState> (event, "CFS");

      // eliminate non-inelastic events and empty events in LHCb
      if (cfs.particles().size() == 0) vetoEvent;

      foreach (const Particle &myp, cfs.particles()){
          double dPV = getPVDCA(event, myp);
          // if IP > 200 microns the particle is not considered prompt
          if ((dPV < 0.) || (dPV > 0.2*millimeter)) continue;
          // histo gets filled only for inelastic events (at least one prompt charged particle)
          _hInelasticXs->fill(sqrtS(), weight);
          break;
      } //end loop on particles

    }


    /// Normalise histograms etc., after the run
    void finalize() {
      scale(_hInelasticXs, crossSection()/sumOfWeights()/millibarn);
    }

    //@}


  private:



    /*
     * Find primary vertex for a given HepMC::GenEvent
     */
    const HepMC::GenVertex* getPrimaryVertex(const Event& evt) {
      const pair<HepMC::GenParticle*, HepMC::GenParticle*> &gpBeam = evt.genEvent()->beam_particles();
      // Test that beam particle are protons and dissociate at the same position in space
      if ( ( gpBeam.first->pdg_id() != 2212 ) || ( gpBeam.second->pdg_id() != 2212 ) ) return 0;
      if ( gpBeam.first->end_vertex()->barcode() == gpBeam.second->end_vertex()->barcode() ) return 0;
      if ( (0 == gpBeam.first->end_vertex()) || (0 == gpBeam.second->end_vertex()) ) return 0;
      if ( gpBeam.first->end_vertex()->position() != gpBeam.second->end_vertex()->position() ) return 0;
      return gpBeam.first->end_vertex();
    }

    /*
     * Compute Distance of Closest Approach in z range for one particle. Assuming length unit is mm.
     * Returns -1. if unable to compute the DCA to PV.
     */
    double getPVDCA(const Event& evt, const Particle& pp) {
      const HepMC::GenVertex* pv = getPrimaryVertex(evt);
      const HepMC::GenVertex* ivtx = pp.genParticle()->production_vertex();
      if ( (0 == pv) || (0 == ivtx) ) return -1.;
      double vt = pv->position().t();
      double vx = pv->position().x();
      double vy = pv->position().y();
      double vz = pv->position().z();
      const FourVector vppv(vt, vx, vy, vz);
      vt = ivtx->position().t();
      vx = ivtx->position().x();
      vy = ivtx->position().y();
      vz = ivtx->position().z();
      const FourVector vpivtx(vt, vx, vy, vz);
      const FourMomentum& mom = pp.momentum();
      const Vector3 versor(multiply(mom.p3(), 1./mom.p3().mod()));
      const Vector3 diffPV(subtract(vpivtx.vector3(),vppv.vector3()));
      MSG_DEBUG("Momentum unit vector: (x,y,z) = (" << versor.x() << ", " << versor.y() << ", " << versor.z() << ")");
      Vector3 dist(diffPV);
      double vProjMom = dot(diffPV, versor);
      dist -= multiply(versor, vProjMom);
      MSG_DEBUG("Distance PV to particle momentum: (x,y,z) = (" << dist.x() << ", " << dist.y() << ", " << dist.z() << ")");
      MSG_DEBUG("Distance between PV and particle trajectory: " << dist.mod() * millimeter << " mm.");
      return dist.mod() * millimeter;
    }


    /// @name Histograms
    //@{
    Histo1DPtr _hInelasticXs;

    double _etaMin;
    double _etaMax;
    double _ptMin;

    //@}

  };



  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(LHCB_2015_I1333223);

}
